use std::{path::PathBuf, str::FromStr};

use crate::{core, server::LSPServer};
#[allow(unused)]
use log::info;
use tower_lsp::lsp_types::*;

use super::to_escape_path;

pub fn inlay_hint(server: &LSPServer, params: &InlayHintParams) -> Option<Vec<InlayHint>> {
    let uri = &params.text_document.uri;
    let path = PathBuf::from_str(uri.path()).unwrap();
    let path = to_escape_path(&path);
    let path_string = path.to_str().unwrap();
    let visible_range = core::hdlparam::Range::from_lsp_range(&params.range);

    info!("enter hints");

    let fast_map = server.srcs.hdl_param.path_to_hdl_file.read().unwrap();
    // 先找到 当前 所在的 hdlfile
    if let Some(hdl_file) = &fast_map.get(path_string) {
        info!("enter some");
        
        let fast = &hdl_file.fast;
        let mut hints = Vec::<InlayHint>::new();
        // 制作例化模块的 hint
        for module in &fast.content {
            for instance in &module.instances {
                // 根据在可见视图外面的 range 就不管了
                if is_visible_range(&instance.instparams, &visible_range) {
                    hints.extend(make_instparam_hints(params, instance));
                }

                if is_visible_range(&instance.instports, &visible_range) {
                    hints.extend(make_instport_hints(params, instance));
                }
            }
        }

        return Some(hints);
    }

    None
}

fn is_visible_range(
    target_range: &Option<core::hdlparam::Range>,
    visible_range: &core::hdlparam::Range
) -> bool {
    if let Some(target_range) = target_range {
        if target_range.before(visible_range) || target_range.after(visible_range) {
            return false;
        }
        return true;
    }
    false
}

fn make_instparam_hints(
    params: &InlayHintParams,
    instance: &core::hdlparam::Instance
) -> Vec<InlayHint> {
    let mut hints = Vec::<InlayHint>::new();


    hints
}

fn make_instport_hints(
    params: &InlayHintParams,
    instance: &core::hdlparam::Instance
) -> Vec<InlayHint> {
    let mut hints = Vec::<InlayHint>::new();
    for port_assigment in &instance.intstport_assignments {
        let port_desc = MarkupContent {
            kind: MarkupKind::Markdown,
            value: format!("```verilog\n{:?}\n```", port_assigment.port)
        };

        let hint = InlayHint {
            position: port_assigment.range.to_lsp_range().start,
            label: InlayHintLabel::String("start".to_string()),
            padding_left: Some(true),
            padding_right: Some(true),
            kind: Some(InlayHintKind::PARAMETER),
            text_edits: None,
            tooltip: Some(InlayHintTooltip::MarkupContent(port_desc)),
            data: None
        };
        hints.push(hint);

        let port_desc = MarkupContent {
            kind: MarkupKind::Markdown,
            value: format!("```verilog\n{:?}\n```", port_assigment.port)
        };
        let hint = InlayHint {
            position: port_assigment.range.to_lsp_range().end,
            label: InlayHintLabel::String("end".to_string()),
            padding_left: Some(true),
            padding_right: Some(true),
            kind: Some(InlayHintKind::PARAMETER),
            text_edits: None,
            tooltip: Some(InlayHintTooltip::MarkupContent(port_desc)),
            data: None
        };
        hints.push(hint);
    }

    hints
}